import java.util.*;
import org.w3c.tools.resources.ResourceReference;

//Manual cflowdepth
aspect CflowDepth {

	public static ThreadLocal cflowdepth = new ThreadLocal() {
		protected synchronized Object initialValue() {
			return new Integer(0);
		}
	};

	before() : execution(* *(..)) && cflow(Common.lockCallers()) && 
				!within(CflowDepth || LockUpdater || LockChecker ||
						Logger || MemQueryServer || memquery.*){
		//System.out.println("Entering " + thisJoinPoint.getSignature());
		Integer prev = (Integer) cflowdepth.get();
		cflowdepth.set(new Integer(prev.intValue() + 1));
		//System.out.println("level " + cflowdepth.get());
	}

	after() : execution(* *(..)) && cflow(Common.lockCallers()) &&
				!within(CflowDepth || LockUpdater || LockChecker ||
						Logger || MemQueryServer || memquery.*){
		//System.out.println("Exiting " + thisJoinPoint.getSignature());
		Integer prev= (Integer) cflowdepth.get();
		cflowdepth.set(new Integer(prev.intValue() - 1));
		//System.out.println("level " + cflowdepth.get());
	}
}

aspect LockUpdater {

	public static ThreadLocal depthMap = new ThreadLocal() {
		protected synchronized Object initialValue() {
			 return new HashMap(); /*<Integer, Set<ResourceReference>>*/
		}
	};
	
	pointcut lock(org.w3c.tools.resources.ResourceReference r) :
		Logger.lock() && target(r);

	pointcut unlock(org.w3c.tools.resources.ResourceReference r) :
		Logger.unlock() && target(r);



	before(org.w3c.tools.resources.ResourceReference r) : lock(r) {
		HashMap map = (HashMap) depthMap.get();
		Integer depth = (Integer)CflowDepth.cflowdepth.get();
		List depthList = (List) map.get(depth);
		if (depthList == null) {
			depthList = new LinkedList();
			map.put(depth, depthList);
		}
		//System.out.println("Adding " + r + "at level " + depth);
		depthList.add(r);
	}

	before(org.w3c.tools.resources.ResourceReference r) : unlock(r) {
		HashMap map = (HashMap) depthMap.get();
		Integer depth = (Integer)CflowDepth.cflowdepth.get();
		List depthList = (List) map.get(depth);
		if (depthList == null || !depthList.contains(r)) {
			try {
				throw new Exception ("Unmatched unlock");
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		//System.out.println("Removing " + r + "at level " + depth);
		depthList.remove(r);
	}

}

aspect LockChecker {
	declare precedence : LockUpdater, CflowDepth, LockChecker;

	after() : Common.lockCallers() {
		HashMap map = (HashMap) LockUpdater.depthMap.get();
		Integer depth = (Integer)CflowDepth.cflowdepth.get();
		List depthList = (List) map.get(depth);
		if (depthList != null && depthList.size() > 0) {
			try {
				throw new Exception ("Unmatched lock at level " + depth);
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
		if (depthList != null) {
			depthList.clear();
		}
	}
}
