/* abc - The AspectBench Compiler
 * Copyright (C) 2005
 *
 * This compiler is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This compiler is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this compiler, in the file LESSER-GPL;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
package aspectdrawchange;

import org.jhotdraw.standard.*;
import org.jhotdraw.framework.*;
import java.util.*;
/**
 * @author Neil Ongkingco
 *
 */
public privileged aspect AspectDrawChangeNotify {
    
    private boolean debug = false;
    private void debPrint(String s) {
        if (debug) {
            System.out.println(s);
        }
    }
    
    //--------------------------------------------------------------------
    //Pointcut definitions
    pointcut addDrawListener(
            org.jhotdraw.framework.DrawingChangeListener listener,
            org.jhotdraw.standard.StandardDrawing object):
        execution (* org.jhotdraw.standard.StandardDrawing+.addDrawingChangeListener(org.jhotdraw.framework.DrawingChangeListener+)) 
        && this(object) && args(listener);

    pointcut figureInvalidated(org.jhotdraw.framework.FigureChangeEvent event,
            org.jhotdraw.standard.StandardDrawing object) :
        execution (* org.jhotdraw.standard.StandardDrawing+.figureInvalidated(
                org.jhotdraw.framework.FigureChangeEvent)) &&
        this(object) && args(event);

    pointcut fireDrawingTitleChanged(
            org.jhotdraw.standard.StandardDrawing object) :
        execution (* org.jhotdraw.standard.StandardDrawing+.fireDrawingTitleChanged()) &&
        this(object);

    pointcut figureRequestUpdate(org.jhotdraw.framework.FigureChangeEvent event,
            org.jhotdraw.standard.StandardDrawing object) :
        execution (* org.jhotdraw.standard.StandardDrawing+.figureRequestUpdate(
                org.jhotdraw.framework.FigureChangeEvent)) &&
        this(object) && args(event);

	after(DrawingChangeListener listener, StandardDrawing object) :
		addDrawListener(listener, object) {
		//don't need to do anything, since standard drawing still stores its
		//listeners.
	}
	
	after(FigureChangeEvent event, StandardDrawing object) :
		figureInvalidated(event,object) {
		Iterator iter = object.drawingChangeListeners();
		if (iter != null) {
			while(iter.hasNext()) {
				DrawingChangeListener l = (DrawingChangeListener)iter.next();
				l.drawingInvalidated(new DrawingChangeEvent(object, event.getInvalidatedRectangle()));
			}
		}
	}

	after(StandardDrawing object) :
		fireDrawingTitleChanged(object) {
		Iterator iter = object.drawingChangeListeners();
		if (iter != null) {
			while (iter.hasNext()) {
				DrawingChangeListener l = (DrawingChangeListener)iter.next();
				l.drawingTitleChanged(new DrawingChangeEvent(object, null));
			}
		}
	}
	
	after(FigureChangeEvent event, StandardDrawing object) :
		figureRequestUpdate(event, object) {
		Iterator iter = object.drawingChangeListeners();
		if (iter != null) {
			while (iter.hasNext()) {
				DrawingChangeListener l = (DrawingChangeListener)iter.next();
				l.drawingRequestUpdate(new DrawingChangeEvent(object, null));
			}
		}
	}
}
