import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.IdentityHashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.apache.commons.collections.map.*;

/**
 * Native AspectJ aspect for the "easy" hashcode benchmark (the one which works on hashmaps of *collecitons*)
 * 
 * @author Eric Bodden
 */
public aspect HashSetNativeAspect {

    pointcut addCollection(HashSet s, Collection c): call(* Collection.add(Object)) && target(s) && args(c);
    pointcut removeCollection(HashSet s, Collection c): call(* Collection.remove(Object)) && target(s) && args(c);
    pointcut modifyCollection(Collection c): (call(* Collection.add(..)) || call(* Collection.addAll(..)) || call(* Collection.remove(..)) || call(* Collection.removeAll(..))) && target(c);
    pointcut scope(): !within(HashSetNativeAspect); 
    
    WeakIdentityHashMap collectionsToItsHashSets = new WeakIdentityHashMap();
    
    after(HashSet s, Collection c) returning ():
        addCollection(s,c) && scope() {
        if(collectionsToItsHashSets.get(c)==null) {
            collectionsToItsHashSets.put(c,new WeakIdentityHashSet());
        }
        WeakIdentityHashSet sets = (WeakIdentityHashSet)collectionsToItsHashSets.get(c);
        if(sets!=null) {
	        sets.add(s);
        }
    }
    
    after(HashSet s, Collection c) returning ():
        removeCollection(s,c) && scope() {
        WeakIdentityHashSet sets = (WeakIdentityHashSet)collectionsToItsHashSets.get(c);
        if(sets!=null) {
            sets.remove(s);
        }
    }
    
    before(Collection c):
        modifyCollection(c) && scope() {       
        WeakIdentityHashSet sets = (WeakIdentityHashSet)collectionsToItsHashSets.get(c);
        if(sets!=null) {
            for (Iterator iter = sets.iterator(); iter.hasNext();) {
                HashSet h = (HashSet) iter.next();
                System.out.println("Modified collection "+c+", which is contained in HashSet."+
                "Next call to HashSet.contains(Object) on "+h+" may return false results.");
            }
        }
    }
    
}

class Test {
    
    public static void main(String[] args) {
        Set s = new HashSet();
        Collection c = new ArrayList();
        s.add(c);
        s.remove(c);
        System.out.println("this is ok");
        c.add("this is ok");
        s.add(c);
        System.out.println("don't do this");
        c.add("don't do this");        
    }

}
