package handoptim;

import java.util.*;
import java.lang.ref.WeakReference;

// roll our own vectors, with a special field for
// keeping track of enumerations
class MyVector extends Vector {
   long stamp = 0;
   public MyVector() { super(); }
   public MyVector(int n) { super(n); }
}

public aspect FailSafeEnum {

        pointcut vector_update() :
	call(* Vector.add*(..)) ||
	call(* Vector.clear()) ||
        call(* Vector.insertElementAt(..)) ||
        call(* Vector.remove*(..)) ||
        call(* Vector.retainAll(..)) ||
        call(* Vector.set*(..));

	/* --------------------------------------------------
           Vector is not weavable, so we replace all vectors
           in JHotDraw by our own subclass. This requires that
           we know that no Vector is constructed outside the
           weavable classes... check this as follows: */

	declare warning : 
	   call(Vector (!CH..*).*(..)) :
	    "Vector constructed outside JHotDraw";



        // intercept all Vector constructor calls and use our own
        Vector around(int n) : call(Vector.new(int)) && args(n) 
                               && !within(FailSafeEnum) {
            return new MyVector(n);
        }

	// also without arguments 
	Vector around() : call(Vector.new()) && !within(FailSafeEnum) {
            return new MyVector();
        }    
        
	/* end of hack to make Vector weavable
           ----------------------------------------------------- */



        /* Enumeration is not weavable, so we introduce a dummy interface */
        /* crude check to see whether that is possible: */

	declare warning : call(((!CH..*) && Enumeration+).new(Vector)) :
	        "Constructor call on enumeration outside JHotDraw";
	
        public interface MyEnumeration extends Enumeration {}
        /* and put that on everything that implements Enumeration already */
	declare parents : Enumeration+ implements MyEnumeration;

        // tag all enumerations with stamp and datasource 
        private long MyEnumeration.stamp;
        private void MyEnumeration.setStamp(long n) { stamp = n; }
        private long MyEnumeration.getStamp() { return stamp; }
        private MyVector MyEnumeration.vector;
        private void MyEnumeration.setVector(MyVector v) { vector = v;}
        private MyVector MyEnumeration.getVector() {return vector;}
        

        synchronized after(MyVector ds) returning(MyEnumeration e) :
	    call(Enumeration+.new(..)) && args(ds) {
            e.setStamp(ds.stamp);
            e.setVector(ds);
           
        }

        synchronized after(MyVector ds) :
            vector_update() && target(ds) {
            ds.stamp++;
        }
        
	synchronized before(MyEnumeration e) :          
	    call(Object Enumeration.nextElement()) &&
            target(e) {
	    if (e.getStamp() != e.getVector().stamp)
		{throw new ConcurrentModificationException();}
        }
                
     
}
   
