import org.apache.commons.collections.map.*;
import java.util.*;
import org.aspectj.lang.*;
import org.aspectj.lang.reflect.*;

public aspect NullTrack
{
    /* ref -> (Sig -> SourceLocation) */
    Map refs = new ReferenceIdentityMap(AbstractReferenceMap.WEAK,
                                        AbstractReferenceMap.HARD,
                                        true);

    int lineno = -1;
    Set null_fields;

    class NullField
    {
        public String signature;
        public String set_loc;

        public NullField(String signature, String set_loc)
        {
            this.signature = signature.intern();
            this.set_loc = set_loc.intern();
        }

        public int hashCode()
        {
            return signature.hashCode();
        }

        public boolean equals(Object other)
        {
            if (other instanceof NullField) {
                NullField otherNF = (NullField) other;

                return signature == otherNF.signature
                    && set_loc == otherNF.set_loc;
            }
            return false;
        }

        public String toString()
        {
            return "[" + signature + ", " + set_loc + "]";
        }
    }


    pointcut anyset(Object t, Object o) :
        set(* *) && args(o) && target(t) && !within(NullTrack);

    pointcut anyget(Object t) :
        get(* *) && target(t) && !within(NullTrack);
   
    String sig(JoinPoint jp) {
        return jp.getSignature().toString();
    }

    String loc(JoinPoint jp) {
        return jp.getSourceLocation().toString();
    }
 
    int lineno(JoinPoint jp) {
        return jp.getSourceLocation().getLine();
    }

    before(Object o,Object t) : 
        anyset(t,o)
    {
        String signature = sig(thisJoinPoint);
        String set_loc = loc(thisJoinPoint);

        Map fields = (Map) refs.get(t);

        if (fields == null) {
            if (o != null)
                return;

            fields = new HashMap();
            refs.put(t, fields);
        }

        if (o != null) {
            if (fields.containsKey(signature))
                fields.remove(signature);

            if (fields.isEmpty())
                refs.remove(t);

            return;
        }

        fields.put(signature, set_loc);
    }

    after(Object t) returning(Object o):
        anyget(t)
    {
        if (o != null)
            return;

        String signature = sig(thisJoinPoint);
        int lineno = lineno(thisJoinPoint);

        if (this.lineno != lineno)
            null_fields = new HashSet();
        this.lineno = lineno;

        Map fields = (Map) refs.get(t);
        String set_loc = (String) fields.get(signature);

        if (set_loc != null)
            null_fields.add(new NullField(signature, set_loc));
    }

    after() throwing(NullPointerException ne) : 
        (call(* *.*(..)) || get(* *) || set(* *))
    {
        int lineno = lineno(thisJoinPoint);
        if (this.lineno != lineno)
            return;

        Iterator i = null_fields.iterator();
        while (i.hasNext()) {
            NullField nf = (NullField) i.next();
            report(thisJoinPoint, nf.signature, nf.set_loc);
        }
    }

    void report(JoinPoint jp, String signature, String set_loc)
    {
        System.out.println("Null pointer exception at " + loc(jp));
        System.out.println("   at source location " + jp.getSourceLocation());
        System.out.println("   possibly caused by null value of " + signature);
        System.out.println("   which was set to null at " + set_loc);
    }
}
