import java.util.Collection;
import java.util.HashSet;

/**
 * Native AspectJ aspect for the "general" hashcode benchmark (the one which works on *any* hashmap)
 * 
 * @author Eric Bodden
 */
public aspect RealHashSetNativeAspect {

    pointcut add(HashSet s, Object o): call(boolean Collection.add(Object)) && target(s) && args(o);
    pointcut remove(HashSet s, Object o): call(boolean Collection.remove(Object)) && target(s) && args(o);
    pointcut contains(HashSet s, Object o): call(* Collection.contains(Object)) && target(s) && args(o);
    
    pointcut scope(): !within(RealHashSetNativeAspect); 

    
    WeakIdentityHashMap map /* hashmap -> (object -> hashcode) */
        = new WeakIdentityHashMap();
    
    after(HashSet s, Object o) returning(): scope() && add(s, o) {
        if(map.get(s)==null) {
            map.put(s, new WeakIdentityHashMap() /* object -> hashcode */);
        }
        WeakIdentityHashMap map2 = (WeakIdentityHashMap)map.get(s); /* object -> hashcode */
        map2.put(o, new Integer(o.hashCode()));
    }
    
    after(HashSet s, Object o) returning(): scope() && remove(s, o) {
        WeakIdentityHashMap map2 = (WeakIdentityHashMap)map.get(s); /* object -> hashcode */
        if(map2!=null) {
            map2.remove(o);
        }
    }
    
    before(HashSet s, Object o): scope() && contains(s, o) {
        if(map.get(s)!=null) {
            WeakIdentityHashMap map2 = (WeakIdentityHashMap)map.get(s); /* object -> hashcode */
            Object code = map2.get(o);
            if(code!=null) {
                if(!( ((Integer)code).intValue()==o.hashCode()) ) {
                    System.out.println("HashCode of "+o+" changed while being in HashSet "+s+"!");
                }
            }
        }
    }
    

}
    
class RealTest {
    
    static class Container {
        String s = null;
        
        public int hashCode() {
            return (s==null) ? 0 : s.hashCode();
        }
    }
    
    public static void main(String[] args) {
        HashSet s = new HashSet();
        Container c = new Container();
        
        s.add(c);
        s.remove(c);
        c.s = "this is ok, cause c is not in s";
        System.out.println("This is ok.");
        s.contains(c);
        
        s.add(c);
        c.s = "this is not ok, cause c is in s";
        System.out.println("This is a fault.");
        s.contains(c);
    }

}
